"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useAxisContext = exports.usePlotDispatchContext = exports.usePlotContext = exports.plotDispatchContext = exports.plotContext = exports.plotReducer = void 0;
const d3_array_1 = require("d3-array");
const d3_scale_1 = require("d3-scale");
const react_1 = require("react");
const utils_1 = require("../utils");
function plotReducer(state, action) {
    switch (action.type) {
        case 'newData': {
            state.series.push(action.payload);
            break;
        }
        case 'removeData': {
            const { id } = action.payload;
            const seriesFiltered = state.series.filter((series) => series.id !== id);
            state.series = seriesFiltered;
            break;
        }
        case 'newAxis': {
            const { id, position, ...values } = action.payload;
            let currentAxis = state.axes[id];
            if (currentAxis) {
                (0, utils_1.validatePosition)(currentAxis.position, position, id);
                state.axes[id] = { ...currentAxis, position, ...values };
            }
            else {
                state.axes[id] = { position, ...values };
            }
            break;
        }
        case 'removeAxis': {
            const { id } = action.payload;
            delete state.axes[id];
            break;
        }
        case 'addHeading': {
            state.headingPosition = action.payload.position;
            break;
        }
        case 'removeHeading': {
            state.headingPosition = null;
            break;
        }
        case 'addLegend': {
            state.legendPosition = action.payload.position;
            state.legendMargin = action.payload.margin;
            break;
        }
        case 'removeLegend': {
            state.legendPosition = null;
            state.legendMargin = 0;
            break;
        }
        default: {
            // eslint-disable-next-line @typescript-eslint/no-explicit-any
            throw new Error(`Unknown reducer type ${action.type}`);
        }
    }
}
exports.plotReducer = plotReducer;
exports.plotContext = (0, react_1.createContext)({
    width: 0,
    height: 0,
    plotWidth: 0,
    plotHeight: 0,
    colorScaler: (0, d3_scale_1.scaleOrdinal)(),
    axisContext: {},
});
exports.plotDispatchContext = (0, react_1.createContext)(() => {
    // No-op
});
function usePlotContext() {
    const context = (0, react_1.useContext)(exports.plotContext);
    if (!context) {
        throw new Error('usePlotContext called outside of Plot context');
    }
    return context;
}
exports.usePlotContext = usePlotContext;
function usePlotDispatchContext() {
    const context = (0, react_1.useContext)(exports.plotDispatchContext);
    if (!context) {
        throw new Error('usePlotDispatchContext called outside of Plot context');
    }
    return context;
}
exports.usePlotDispatchContext = usePlotDispatchContext;
function useAxisContext(state, { plotWidth, plotHeight }) {
    const context = (0, react_1.useMemo)(() => {
        let axisContext = {};
        for (const id in state.axes) {
            const axis = state.axes[id];
            const isHorizontal = ['top', 'bottom'].includes(axis.position);
            const xY = isHorizontal ? 'x' : 'y';
            // Get limits from state or data
            const axisMin = axis.min !== undefined ? axis.min : (0, d3_array_1.min)(state.series, (d) => d[xY].min);
            const axisMax = axis.max !== undefined ? axis.max : (0, d3_array_1.max)(state.series, (d) => d[xY].max);
            // Limits validation
            if (axisMin === undefined || axisMax === undefined) {
                return {};
            }
            if (axisMin > axisMax) {
                throw new Error(`${id}: min (${axisMin}) is bigger than max (${axisMax})`);
            }
            // Limits paddings
            const diff = axisMax - axisMin;
            const minPad = diff * (axis.paddingStart || 0);
            const maxPad = diff * (axis.paddingEnd || 0);
            const range = isHorizontal ? [0, plotWidth] : [plotHeight, 0];
            switch (axis.scale) {
                case 'log': {
                    axisContext[id] = {
                        type: axis.scale,
                        position: axis.position,
                        tickLabelFormat: axis.tickLabelFormat,
                        scale: (0, d3_scale_1.scaleLog)()
                            .domain([axisMin - minPad, axisMax + maxPad])
                            .range(axis.flip ? range.reverse() : range),
                    };
                    break;
                }
                case 'linear':
                default: {
                    axisContext[id] = {
                        type: 'linear',
                        position: axis.position,
                        tickLabelFormat: axis.tickLabelFormat,
                        scale: (0, d3_scale_1.scaleLinear)()
                            .domain([axisMin - minPad, axisMax + maxPad])
                            .range(axis.flip ? range.reverse() : range),
                    };
                    break;
                }
            }
        }
        return axisContext;
    }, [state, plotWidth, plotHeight]);
    return context;
}
exports.useAxisContext = useAxisContext;
